Package: LedPred
Title: Learning from DNA to Predict Enhancers
Description: This package aims at creating a predictive model of regulatory
    sequences used to score unknown sequences based on the content of DNA motifs,
    next-generation sequencing (NGS) peaks and signals and other numerical scores of
    the sequences using supervised classification. The package contains a workflow
    based on the support vector machine (SVM) algorithm that maps features to
    sequences, optimize SVM parameters and feature number and creates a model that
    can be stored and used to score the regulatory potential of unknown sequences.
Version: 1.45.0
Date: 2016-08-13
Author: Elodie Darbo, Denis Seyres, Aitor Gonzalez
Maintainer: Aitor Gonzalez <aitor.gonzalez@univ-amu.fr>
Depends: R (>= 3.2.0), e1071 (>= 1.6)
Imports: akima, ggplot2, irr, jsonlite, parallel, plot3D, plyr, RCurl,
        ROCR, testthat
License: MIT | file LICENSE
LazyData: true
Packaged: 2025-10-30 21:42:00 UTC; biocbuild
biocViews: SupportVectorMachine, Software, MotifAnnotation, ChIPSeq,
        Sequencing, Classification
NeedsCompilation: no
BugReports: https://github.com/aitgon/LedPred/issues
RoxygenNote: 5.0.1
git_url: https://git.bioconductor.org/packages/LedPred
git_branch: devel
git_last_commit: f513d31
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
