% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Data.R
\docType{data}
\name{crm.features}
\alias{crm.features}
\title{This is data to be included in my package}
\description{
This is data to be included in my package
}

